@@ -24,7 +24,6 @@
 
 from __future__ import absolute_import, print_function
 
-import apt
 import apt_pkg
 import copy
 from hashlib import md5
@@ -48,10 +47,6 @@ try:
 except ImportError:
   from ConfigParser import ConfigParser
 from gettext import gettext as _
-try:
-  from urllib.parse import urlparse
-except ImportError:
-  from urlparse import urlparse
 
 import aptsources
 import aptsources.distro
@@ -59,7 +54,16 @@ import softwareproperties
 
 from .AptAuth import AptAuth
 from aptsources.sourceslist import SourcesList, SourceEntry
-from .ppa import AddPPASigningKeyThread, expand_ppa_line
+from . import shortcuts
+from . import ppa
+from . import cloudarchive
+
+_SHORTCUT_FACTORIES = [
+    ppa.shortcut_handler,
+    cloudarchive.shortcut_handler,
+    shortcuts.shortcut_handler,
+]
+
 
 class SoftwareProperties(object):
 
@@ -488,19 +492,21 @@ class SoftwareProperties(object):
     """Render a nice output to show the source in a treeview"""
     if source.template == None:
         if source.comment:
-            contents = "<b>%s</b> (%s %s" % (escape(source.comment),
-                                              source.uri, source.dist)
+            contents = "<b>%s</b> %s" % (escape(source.comment).strip(),
+                                         source.dist)
             # Only show the components if there are more than one
             if len(source.comps) > 1:
                 for c in source.comps:
                     contents += " %s" % c
-            contents += ")"
+            if source.type in ("deb-src", "rpm-src"):
+                contents += " %s" % _("(Source Code)")
+            contents += "\n%s" % source.uri
         else:
             contents = "<b>%s %s</b>" % (source.uri, source.dist)
             for c in source.comps:
                 contents += " %s" % c
-        if source.type in ("deb-src", "rpm-src"):
-            contents += " %s" % _("(Source Code)")
+            if source.type in ("deb-src", "rpm-src"):
+                contents += " %s" % _("(Source Code)")
         return contents
     else:
         # try to make use of a corresponding template
@@ -648,7 +654,7 @@ class SoftwareProperties(object):
     helper that checks if a given line is in the source list
     return the channel name or None if not found
     """
-    srcentry = SourceEntry(srcline)    
+    srcentry = SourceEntry(srcline)
     if os.path.exists(self.CHANNEL_PATH):
       for f in glob.glob("%s/*.list" % self.CHANNEL_PATH):
         for line in open(f):
@@ -659,23 +665,30 @@ class SoftwareProperties(object):
     return None
 
   def check_and_add_key_for_whitelisted_channels(self, srcline):
+    # This is maintained for any legacy callers
+    return self.check_and_add_key_for_whitelisted_shortcut(shortcut_handler(srcline))
+
+  def check_and_add_key_for_whitelisted_shortcut(self, shortcut):
     """
     helper that adds the gpg key of the channel to the apt
     keyring *if* the channel is in the whitelist
     /usr/share/app-install/channels or it is a public Launchpad PPA.
     """
+    (srcline, _fname) = shortcut.expand(codename=self.distro.codename)
     channel = self._is_line_in_whitelisted_channel(srcline)
     if channel:
       keyp = "%s/%s.key" % (self.CHANNEL_PATH, channel)
       self.add_key(keyp)
-    # FIXME: abstract this all alway into the ppa.py file
-    parsed_uri = urlparse(SourceEntry(srcline).uri)
-    if parsed_uri.netloc == 'ppa.launchpad.net':
-      worker = AddPPASigningKeyThread(parsed_uri.path, self.options and self.options.keyserver)
-      worker.start()
-      return worker
-    else:
-      return None
+
+    cdata = (shortcut.add_key, {'keyserver': (self.options and
+                                              self.options.keyserver)})
+    def addkey_func():
+        func, kwargs = cdata
+        func(**kwargs)
+
+    worker = threading.Thread(target=addkey_func)
+    worker.start()
+    return worker
 
   def update_interface(self):
     " abstract interface to keep the UI alive "
@@ -699,10 +712,18 @@ class SoftwareProperties(object):
 
   def add_source_from_line(self, line, enable_source_code=False):
     """
-    Add a source with the given apt line and auto-add
-    signing key if we have it in the whitelist
+    Add a source for the given line.
     """
-    (deb_line, file) = expand_ppa_line(line.strip(), self.distro.codename)
+    return self.add_source_from_shortcut(shortcut=shortcut_handler(line),
+                                         enable_source_code=enable_source_code)
+
+  def add_source_from_shortcut(self, shortcut, enable_source_code=False):
+    """
+    Add a source with the given shortcut and add the signing key if the
+    site is in whitelist or the shortcut implementer adds it.
+    """
+
+    (deb_line, file) = shortcut.expand(codename=self.distro.codename)
     deb_line = self.expand_http_line(deb_line)
     debsrc_entry_type = 'deb-src' if enable_source_code else '# deb-src'
     debsrc_line = debsrc_entry_type + deb_line[3:]
@@ -710,7 +731,7 @@ class SoftwareProperties(object):
     new_debsrc_entry = SourceEntry(debsrc_line, file)
     if new_deb_entry.invalid or new_debsrc_entry.invalid:
       return False
-    worker = self.check_and_add_key_for_whitelisted_channels(deb_line)
+    worker = self.check_and_add_key_for_whitelisted_shortcut(shortcut)
     self.sourceslist.add(new_deb_entry.type,
                          new_deb_entry.uri,
                          new_deb_entry.dist,
@@ -810,6 +831,15 @@ class SoftwareProperties(object):
       return False
 
 
+def shortcut_handler(shortcut):
+    for factory in _SHORTCUT_FACTORIES:
+        ret = factory(shortcut)
+        if ret is not None:
+            return ret
+
+    raise shortcuts.ShortcutException("Unable to handle input '%s'" % shortcut)
+
+
 if __name__ == "__main__":
   sp = SoftwareProperties()
   print(sp.get_release_upgrades_policy())
