@@ -34,8 +34,10 @@ import subprocess
 from aptdaemon import client
 from aptdaemon.errors import NotAuthorizedError, TransactionFailed
 import logging
+import threading
+import sys
 
-from gi.repository import GObject, Gdk, Gtk, Gio
+from gi.repository import GObject, Gdk, Gtk, Gio, GLib
 
 from .SimpleGtkbuilderApp import SimpleGtkbuilderApp
 from .DialogAdd import DialogAdd
@@ -51,6 +53,9 @@ import softwareproperties.SoftwareProperties
 
 from UbuntuDrivers import detect
 
+if GLib.pyglib_version < (3, 9, 1):
+    GLib.threads_init()
+
 (LIST_MARKUP, LIST_ENABLED, LIST_ENTRY_OBJ) = range(3)
 
 (
@@ -119,8 +124,6 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
                     self.window_main.set_transient_for(toplevel)
                 except:
                     pass
-        if options and options.open_tab:
-            self.notebook_main.set_current_page(int(options.open_tab))
 
         # gsettings
         all_schemas = Gio.Settings.list_schemas()
@@ -154,7 +157,7 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
         # used to store the handlers of callbacks
         self.handlers = {}
 
-        self.apt_cache = apt.Cache()
+        self.apt_cache = {}
         self.apt_client = client.AptClient()
 
         # Put some life into the user interface:
@@ -173,9 +176,14 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
         self.show_distro()
         # Setup and show the Additonal Drivers tab
         self.init_drivers()
-        self.show_drivers()
 
+        # Connect to switch-page before setting initial tab. Otherwise the
+        # first switch goes unnoticed.
         self.notebook_main.connect("switch-page", self.on_main_notebook_page_switched)
+
+        if options and options.open_tab:
+            self.notebook_main.set_current_page(int(options.open_tab))
+
         # Show the import/replace sources.list dialog if a file different
         # to the default sources.list was specified
         # NOTE: If the file path points to the default sources.list the user
@@ -192,6 +200,8 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
         # On the additional drivers page, don't show the backend revert button.
         if page == self.vbox_drivers:
             self.button_revert.set_visible(False)
+            if not self.detect_called:
+                GLib.idle_add(lambda: threading.Thread(target=self.detect_drivers).start())
         else:
             self.button_revert.set_visible(True)
 
@@ -1105,11 +1115,18 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
         self.box_driver_action.pack_end(self.button_driver_restart, False, False, 0)
         self.box_driver_action.pack_end(self.button_driver_cancel, False, False, 0)
 
+        props = {"halign":Gtk.Align.CENTER, "valign":Gtk.Align.CENTER,
+                 "vexpand":True, "visible":True}
+        self.label_driver_detail = Gtk.Label(_("Searching for available "
+                                               "drivers..."), **props)
+        self.box_driver_detail.add(self.label_driver_detail)
+
         self.progress_bar = Gtk.ProgressBar()
         self.box_driver_action.pack_end(self.progress_bar, False, False, 0)
         self.progress_bar.set_visible(False)
 
-        self.devices = detect.system_device_drivers()
+        self.devices = {}
+        self.detect_called = False
         self.driver_changes = []
         self.orig_selection = {}
         # HACK: the case where the selection is actually "Do not use"; is a little
@@ -1121,6 +1138,24 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
         self.nonfree_drivers = 0
         self.ui_building = False
 
+    def detect_drivers(self):
+        # WARNING: This is run in a separate thread.
+        self.detect_called = True
+        try:
+            self.apt_cache = apt.Cache()
+            self.devices = detect.system_device_drivers(self.apt_cache)
+        except:
+            # Catch all exceptions and feed them to apport.
+            GLib.idle_add(self.label_driver_detail.set_text,
+                          _("An error occurred while searching for drivers."))
+
+            # For apport to catch this exception. See
+            # http://bugs.python.org/issue1230540
+            sys.excepthook(*sys.exc_info())
+            return
+
+        GLib.idle_add(self.show_drivers)
+
     def on_driver_selection_changed(self, button, modalias, pkg_name=None):
         if self.ui_building:
             return
@@ -1274,6 +1309,12 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
         return (overall_status, icon, returned_drivers)
 
     def show_drivers(self):
+        if not self.devices:
+            # No drivers found.
+            self.label_driver_detail.set_text(_("No additional drivers available."))
+            return
+        else:
+            self.box_driver_detail.remove(self.label_driver_detail)
         self.ui_building = True
         self.dynamic_device_status = {}
         for device in sorted(self.devices.keys()):
